module Main where

import Data.Semigroup ((<>))
import qualified Data.Text as Text
import Options.Applicative
import System.Environment (getArgs)

import PUBMED (getMetadataWith, getMetadataWithC)
import PUBMED.Types


data MetaParams = MetaParams
  { mp_mAPIKey :: Maybe String
  , mp_term    :: String }

data FetchParams = FetchParams
  { fp_mAPIKey :: Maybe String
  , fp_limit   :: Int
  , fp_term    :: String }

data Command
  = Meta MetaParams
  | Fetch FetchParams

metaParams :: Parser Command
metaParams = do
  api_key <- optional (strOption
    ( long "api-key"
    <> metavar "API_KEY"
    <> help "Pubmed API key"))
  term <- strArgument (metavar "term")
  pure $ Meta (MetaParams api_key term)

fetchParams :: Parser Command
fetchParams = do
  api_key <- optional (strOption
    ( long "api-key"
    <> metavar "API_KEY"
    <> help "Pubmed API key"))
  limit <- option auto
    ( long "limit"
    <> help "Maximum number of documents"
    <> showDefault
    <> value 100
    <> metavar "INT" )
  term <- strArgument (metavar "term")

  pure $ Fetch (FetchParams api_key limit term)

params :: Parser Command
params = metaParams <|> fetchParams

opts :: ParserInfo Command
opts = info (params <**> helper)
       (fullDesc
        <> progDesc "A program to test PUBMD"
         <> header "crawlerPubMed-exe")

main :: IO ()
main = run =<< execParser opts
  where

run :: Command -> IO ()
run (Meta (MetaParams { mp_mAPIKey, mp_term })) = do
  let config = Config { mAPIKey = Text.pack <$> mp_mAPIKey }

  eDocs <- getMetadataWithC config (Text.pack mp_term) Nothing
  case eDocs of
    Left err -> print err
    Right (count, _docs) -> print $ show count
run (Fetch (FetchParams { fp_mAPIKey, fp_term, fp_limit })) = do
  let config = Config { mAPIKey = Text.pack <$> fp_mAPIKey }

  eDocs <- getMetadataWithC config (Text.pack fp_term) (Just $ fromIntegral fp_limit)
  case eDocs of
    Left err -> print err
    Right (count, docs) -> print $ show count
